@@ -5,7 +5,7 @@
 
 #include <string.h> // because Borland's STL is braindead, we have to include <string.h> _before_ <string> in order to get memcpy
 
-#include "common.hpp"
+#include "test.hpp"
 
 #include "writer_string.hpp"
 
@@ -18,6 +18,10 @@
 #include <string>
 #include <algorithm>
 
+#ifndef PUGIXML_NO_EXCEPTIONS
+#	include <stdexcept>
+#endif
+
 #ifdef __MINGW32__
 #	include <io.h> // for unlink in C++0x mode
 #endif
@@ -26,6 +30,8 @@
 #	include <unistd.h> // for unlink
 #endif
 
+using namespace pugi;
+
 static bool load_file_in_memory(const char* path, char*& data, size_t& size)
 {
 	FILE* file = fopen(path, "rb");
@@ -60,30 +66,30 @@ static bool test_file_contents(const char* path, const char* data, size_t size)
 
 TEST(document_create_empty)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 	CHECK_NODE(doc, STR(""));
 }
 
 TEST(document_create)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 	doc.append_child().set_name(STR("node"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 #ifndef PUGIXML_NO_STL
 TEST(document_load_stream)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	std::istringstream iss("<node/>");
 	CHECK(doc.load(iss));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_stream_offset)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	std::istringstream iss("<foobar> <node/>");
 
@@ -91,52 +97,66 @@ TEST(document_load_stream_offset)
 	iss >> s;
 
 	CHECK(doc.load(iss));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_stream_text)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	std::ifstream iss("tests/data/multiline.xml");
 	CHECK(doc.load(iss));
-	CHECK_NODE(doc, STR("<node1 /><node2 /><node3 />"));
+	CHECK_NODE(doc, STR("<node1/><node2/><node3/>"));
 }
 
 TEST(document_load_stream_error)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	std::ifstream fs("filedoesnotexist");
 	CHECK(doc.load(fs).status == status_io_error);
-	
+}
+
+TEST(document_load_stream_out_of_memory)
+{
+	xml_document doc;
+
 	std::istringstream iss("<node/>");
 	test_runner::_memory_fail_threshold = 1;
 	CHECK_ALLOC_FAIL(CHECK(doc.load(iss).status == status_out_of_memory));
 }
 
+TEST(document_load_stream_wide_out_of_memory)
+{
+	xml_document doc;
+
+	std::basic_istringstream<wchar_t> iss(L"<node/>");
+	test_runner::_memory_fail_threshold = 1;
+	CHECK_ALLOC_FAIL(CHECK(doc.load(iss).status == status_out_of_memory));
+}
+
 TEST(document_load_stream_empty)
 {
 	std::istringstream iss;
 
-	pugi::xml_document doc;
+	xml_document doc;
 	doc.load(iss); // parse result depends on STL implementation
 	CHECK(!doc.first_child());
 }
 
 TEST(document_load_stream_wide)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	std::basic_istringstream<wchar_t> iss(L"<node/>");
 	CHECK(doc.load(iss));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 #ifndef PUGIXML_NO_EXCEPTIONS
 TEST(document_load_stream_exceptions)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	// Windows has newline translation for text-mode files, so reading from this stream reaches eof and sets fail|eof bits.
 	// This test does not cause stream to throw an exception on Linux - I have no idea how to get read() to fail except
@@ -159,7 +179,7 @@ TEST(document_load_stream_exceptions)
 
 TEST(document_load_stream_error_previous)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 	CHECK(doc.load_string(STR("<node/>")));
 	CHECK(doc.first_child());
 
@@ -170,7 +190,7 @@ TEST(document_load_stream_error_previous)
 
 TEST(document_load_stream_wide_error_previous)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 	CHECK(doc.load_string(STR("<node/>")));
 	CHECK(doc.first_child());
 
@@ -186,11 +206,6 @@ public:
     {
         this->setg(begin, begin, end);
     }
-
-    typename std::basic_streambuf<T>::int_type underflow()
-    {
-        return this->gptr() == this->egptr() ? std::basic_streambuf<T>::traits_type::eof() : std::basic_streambuf<T>::traits_type::to_int_type(*this->gptr());
-    }
 };
 
 TEST(document_load_stream_nonseekable)
@@ -199,9 +214,9 @@ TEST(document_load_stream_nonseekable)
     char_array_buffer<char> buffer(contents, contents + sizeof(contents) / sizeof(contents[0]));
     std::istream in(&buffer);
 
-    pugi::xml_document doc;
+    xml_document doc;
     CHECK(doc.load(in));
-    CHECK_NODE(doc, STR("<node />"));
+    CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_stream_wide_nonseekable)
@@ -210,22 +225,22 @@ TEST(document_load_stream_wide_nonseekable)
     char_array_buffer<wchar_t> buffer(contents, contents + sizeof(contents) / sizeof(contents[0]));
     std::basic_istream<wchar_t> in(&buffer);
 
-    pugi::xml_document doc;
+    xml_document doc;
     CHECK(doc.load(in));
-    CHECK_NODE(doc, STR("<node />"));
+    CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_stream_nonseekable_large)
 {
-	std::basic_string<pugi::char_t> str;
+	std::basic_string<char_t> str;
 	str += STR("<node>");
-	for (int i = 0; i < 10000; ++i) str += STR("<node />");
+	for (int i = 0; i < 10000; ++i) str += STR("<node/>");
 	str += STR("</node>");
 
-    char_array_buffer<pugi::char_t> buffer(&str[0], &str[0] + str.length());
-    std::basic_istream<pugi::char_t> in(&buffer);
+    char_array_buffer<char_t> buffer(&str[0], &str[0] + str.length());
+    std::basic_istream<char_t> in(&buffer);
 
-    pugi::xml_document doc;
+    xml_document doc;
     CHECK(doc.load(in));
     CHECK_NODE(doc, str.c_str());
 }
@@ -238,46 +253,238 @@ TEST(document_load_stream_nonseekable_out_of_memory)
 
     test_runner::_memory_fail_threshold = 1;
 
-    pugi::xml_document doc;
+    xml_document doc;
+    CHECK_ALLOC_FAIL(CHECK(doc.load(in).status == status_out_of_memory));
+}
+
+TEST(document_load_stream_wide_nonseekable_out_of_memory)
+{
+    wchar_t contents[] = L"<node />";
+    char_array_buffer<wchar_t> buffer(contents, contents + sizeof(contents) / sizeof(contents[0]));
+    std::basic_istream<wchar_t> in(&buffer);
+
+    test_runner::_memory_fail_threshold = 1;
+
+    xml_document doc;
     CHECK_ALLOC_FAIL(CHECK(doc.load(in).status == status_out_of_memory));
 }
 
 TEST(document_load_stream_nonseekable_out_of_memory_large)
 {
-	std::basic_string<pugi::char_t> str;
-	str += STR("<node>");
-	for (int i = 0; i < 10000; ++i) str += STR("<node />");
-	str += STR("</node>");
+	std::basic_string<char> str;
+	str += "<node>";
+	for (int i = 0; i < 10000; ++i) str += "<node />";
+	str += "</node>";
 
-    char_array_buffer<pugi::char_t> buffer(&str[0], &str[0] + str.length());
-    std::basic_istream<pugi::char_t> in(&buffer);
+    char_array_buffer<char> buffer(&str[0], &str[0] + str.length());
+    std::basic_istream<char> in(&buffer);
 
-    test_runner::_memory_fail_threshold = 10000 * 8 * 3 / 2;
+    test_runner::_memory_fail_threshold = 32768 * 3 + 4096;
 
-    pugi::xml_document doc;
+    xml_document doc;
     CHECK_ALLOC_FAIL(CHECK(doc.load(in).status == status_out_of_memory));
 }
+
+TEST(document_load_stream_wide_nonseekable_out_of_memory_large)
+{
+	std::basic_string<wchar_t> str;
+	str += L"<node>";
+	for (int i = 0; i < 10000; ++i) str += L"<node />";
+	str += L"</node>";
+
+    char_array_buffer<wchar_t> buffer(&str[0], &str[0] + str.length());
+    std::basic_istream<wchar_t> in(&buffer);
+
+    test_runner::_memory_fail_threshold = 32768 * 3 * sizeof(wchar_t) + 4096;
+
+    xml_document doc;
+    CHECK_ALLOC_FAIL(CHECK(doc.load(in).status == status_out_of_memory));
+}
+
+template <typename T> class seek_fail_buffer: public std::basic_streambuf<T>
+{
+public:
+	int seeks;
+
+	seek_fail_buffer(): seeks(0)
+	{
+	}
+
+	typename std::basic_streambuf<T>::pos_type seekoff(typename std::basic_streambuf<T>::off_type, std::ios_base::seekdir, std::ios_base::openmode) PUGIXML_OVERRIDE
+	{
+		seeks++;
+
+		// pretend that our buffer is seekable (this is called by tellg)
+		return seeks == 1 ? 0 : -1;
+	}
+};
+
+TEST(document_load_stream_seekable_fail_seek)
+{
+    seek_fail_buffer<char> buffer;
+    std::basic_istream<char> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+TEST(document_load_stream_wide_seekable_fail_seek)
+{
+    seek_fail_buffer<wchar_t> buffer;
+    std::basic_istream<wchar_t> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+template <typename T> class tell_fail_buffer: public std::basic_streambuf<T>
+{
+public:
+	int seeks;
+
+	tell_fail_buffer(): seeks(0)
+	{
+	}
+
+	typename std::basic_streambuf<T>::pos_type seekoff(typename std::basic_streambuf<T>::off_type, std::ios_base::seekdir dir, std::ios_base::openmode) PUGIXML_OVERRIDE
+	{
+		seeks++;
+
+		return seeks > 1 && dir == std::ios_base::cur ? -1 : 0;
+	}
+
+	typename std::basic_streambuf<T>::pos_type seekpos(typename std::basic_streambuf<T>::pos_type, std::ios_base::openmode) PUGIXML_OVERRIDE
+	{
+		return 0;
+	}
+};
+
+TEST(document_load_stream_seekable_fail_tell)
+{
+    tell_fail_buffer<char> buffer;
+    std::basic_istream<char> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+TEST(document_load_stream_wide_seekable_fail_tell)
+{
+    tell_fail_buffer<wchar_t> buffer;
+    std::basic_istream<wchar_t> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+#ifndef PUGIXML_NO_EXCEPTIONS
+template <typename T> class read_fail_buffer: public std::basic_streambuf<T>
+{
+public:
+    typename std::basic_streambuf<T>::int_type underflow() PUGIXML_OVERRIDE
+	{
+		throw std::runtime_error("underflow failed");
+
+	#ifdef __DMC__
+		return 0;
+	#endif
+	}
+};
+
+TEST(document_load_stream_nonseekable_fail_read)
+{
+    read_fail_buffer<char> buffer;
+    std::basic_istream<char> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+TEST(document_load_stream_wide_nonseekable_fail_read)
+{
+    read_fail_buffer<wchar_t> buffer;
+    std::basic_istream<wchar_t> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+template <typename T> class read_fail_seekable_buffer: public std::basic_streambuf<T>
+{
+public:
+	typename std::basic_streambuf<T>::pos_type offset;
+
+    read_fail_seekable_buffer(): offset(0)
+    {
+    }
+
+    typename std::basic_streambuf<T>::int_type underflow() PUGIXML_OVERRIDE
+	{
+		throw std::runtime_error("underflow failed");
+
+	#ifdef __DMC__
+		return 0;
+	#endif
+	}
+
+	typename std::basic_streambuf<T>::pos_type seekoff(typename std::basic_streambuf<T>::off_type off, std::ios_base::seekdir dir, std::ios_base::openmode) PUGIXML_OVERRIDE
+	{
+		switch (dir)
+		{
+		case std::ios_base::beg: offset = off; break;
+		case std::ios_base::cur: offset += off; break;
+		case std::ios_base::end: offset = 16 + off; break;
+		default: ;
+		}
+		return offset;
+	}
+
+	typename std::basic_streambuf<T>::pos_type seekpos(typename std::basic_streambuf<T>::pos_type pos, std::ios_base::openmode) PUGIXML_OVERRIDE
+	{
+		offset = pos;
+		return pos;
+	}
+};
+
+TEST(document_load_stream_seekable_fail_read)
+{
+    read_fail_seekable_buffer<char> buffer;
+    std::basic_istream<char> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+
+TEST(document_load_stream_wide_seekable_fail_read)
+{
+    read_fail_seekable_buffer<wchar_t> buffer;
+    std::basic_istream<wchar_t> in(&buffer);
+
+    xml_document doc;
+    CHECK(doc.load(in).status == status_io_error);
+}
+#endif
 #endif
 
 TEST(document_load_string)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_string(STR("<node/>")));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_file)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file("tests/data/small.xml"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_file_empty)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file("tests/data/empty.xml").status == status_no_document_element);
 	CHECK(!doc.first_child());
@@ -285,13 +492,13 @@ TEST(document_load_file_empty)
 
 TEST(document_load_file_large)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file("tests/data/large.xml"));
 
-	std::basic_string<pugi::char_t> str;
+	std::basic_string<char_t> str;
 	str += STR("<node>");
-	for (int i = 0; i < 10000; ++i) str += STR("<node />");
+	for (int i = 0; i < 10000; ++i) str += STR("<node/>");
 	str += STR("</node>");
 
 	CHECK_NODE(doc, str.c_str());
@@ -299,7 +506,7 @@ TEST(document_load_file_large)
 
 TEST(document_load_file_error)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file("filedoesnotexist").status == status_file_not_found);
 }
@@ -308,7 +515,7 @@ TEST(document_load_file_out_of_memory)
 {
 	test_runner::_memory_fail_threshold = 1;
 
-	pugi::xml_document doc;
+	xml_document doc;
 	CHECK_ALLOC_FAIL(CHECK(doc.load_file("tests/data/small.xml").status == status_out_of_memory));
 }
 
@@ -316,7 +523,7 @@ TEST(document_load_file_out_of_memory_file_leak)
 {
 	test_runner::_memory_fail_threshold = 1;
 
-	pugi::xml_document doc;
+	xml_document doc;
 
 	for (int i = 0; i < 256; ++i)
 		CHECK_ALLOC_FAIL(CHECK(doc.load_file("tests/data/small.xml").status == status_out_of_memory));
@@ -324,14 +531,14 @@ TEST(document_load_file_out_of_memory_file_leak)
 	test_runner::_memory_fail_threshold = 0;
 
 	CHECK(doc.load_file("tests/data/small.xml"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_file_wide_out_of_memory_file_leak)
 {
 	test_runner::_memory_fail_threshold = 256;
 
-	pugi::xml_document doc;
+	xml_document doc;
 
 	for (int i = 0; i < 256; ++i)
 		CHECK_ALLOC_FAIL(CHECK(doc.load_file(L"tests/data/small.xml").status == status_out_of_memory));
@@ -339,12 +546,12 @@ TEST(document_load_file_wide_out_of_memory_file_leak)
 	test_runner::_memory_fail_threshold = 0;
 
 	CHECK(doc.load_file(L"tests/data/small.xml"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_file_error_previous)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 	CHECK(doc.load_string(STR("<node/>")));
 	CHECK(doc.first_child());
 
@@ -354,19 +561,19 @@ TEST(document_load_file_error_previous)
 
 TEST(document_load_file_wide_ascii)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file(L"tests/data/small.xml"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
-#if !defined(__DMC__) && !defined(__MWERKS__) && !(defined(__MINGW32__) && defined(__STRICT_ANSI__) && !defined(__MINGW64_VERSION_MAJOR))
+#if !defined(__DMC__) && !defined(__MWERKS__) && !(defined(__MINGW32__) && defined(__STRICT_ANSI__) && !defined(__MINGW64_VERSION_MAJOR)) && !defined(__BORLANDC__)
 TEST(document_load_file_wide_unicode)
 {
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_file(L"tests/data/\x0442\x0435\x0441\x0442.xml"));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 #endif
 
@@ -374,22 +581,41 @@ TEST(document_load_file_wide_out_of_memory)
 {
 	test_runner::_memory_fail_threshold = 1;
 
-	pugi::xml_document doc;
+	xml_document doc;
 
-	pugi::xml_parse_result result;
+	xml_parse_result result;
 	result.status = status_out_of_memory;
 	CHECK_ALLOC_FAIL(result = doc.load_file(L"tests/data/small.xml"));
 
 	CHECK(result.status == status_out_of_memory || result.status == status_file_not_found);
 }
 
+#if defined(__linux__) || defined(__APPLE__)
+TEST(document_load_file_special_folder)
+{
+	xml_document doc;
+	xml_parse_result result = doc.load_file(".");
+	// status_out_of_memory is somewhat counter-intuitive but on Linux ftell returns LONG_MAX for directories
+	CHECK(result.status == status_file_not_found || result.status == status_io_error || result.status == status_out_of_memory);
+}
+#endif
+
+#if defined(__linux__)
+TEST(document_load_file_special_device)
+{
+	xml_document doc;
+	xml_parse_result result = doc.load_file("/dev/tty");
+	CHECK(result.status == status_file_not_found || result.status == status_io_error);
+}
+#endif
+
 TEST_XML(document_save, "<node/>")
 {
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_no_declaration | pugi::format_raw, get_native_encoding());
+	doc.save(writer, STR(""), format_no_declaration | format_raw, get_native_encoding());
 
-	CHECK(writer.as_string() == STR("<node />"));
+	CHECK(writer.as_string() == STR("<node/>"));
 }
 
 #ifndef PUGIXML_NO_STL
@@ -397,18 +623,18 @@ TEST_XML(document_save_stream, "<node/>")
 {
 	std::ostringstream oss;
 
-	doc.save(oss, STR(""), pugi::format_no_declaration | pugi::format_raw);
+	doc.save(oss, STR(""), format_no_declaration | format_raw);
 
-	CHECK(oss.str() == "<node />");
+	CHECK(oss.str() == "<node/>");
 }
 
 TEST_XML(document_save_stream_wide, "<node/>")
 {
 	std::basic_ostringstream<wchar_t> oss;
 
-	doc.save(oss, STR(""), pugi::format_no_declaration | pugi::format_raw);
+	doc.save(oss, STR(""), format_no_declaration | format_raw);
 
-	CHECK(oss.str() == L"<node />");
+	CHECK(oss.str() == L"<node/>");
 }
 #endif
 
@@ -417,12 +643,12 @@ TEST_XML(document_save_bom, "<n/>")
 	unsigned int flags = format_no_declaration | format_raw | format_write_bom;
 
 	// specific encodings
-	CHECK(test_save_narrow(doc, flags, encoding_utf8, "\xef\xbb\xbf<n />", 8));
-	CHECK(test_save_narrow(doc, flags, encoding_utf16_be, "\xfe\xff\x00<\x00n\x00 \x00/\x00>", 12));
-	CHECK(test_save_narrow(doc, flags, encoding_utf16_le, "\xff\xfe<\x00n\x00 \x00/\x00>\x00", 12));
-	CHECK(test_save_narrow(doc, flags, encoding_utf32_be, "\x00\x00\xfe\xff\x00\x00\x00<\x00\x00\x00n\x00\x00\x00 \x00\x00\x00/\x00\x00\x00>", 24));
-	CHECK(test_save_narrow(doc, flags, encoding_utf32_le, "\xff\xfe\x00\x00<\x00\x00\x00n\x00\x00\x00 \x00\x00\x00/\x00\x00\x00>\x00\x00\x00", 24));
-	CHECK(test_save_narrow(doc, flags, encoding_latin1, "<n />", 5));
+	CHECK(test_save_narrow(doc, flags, encoding_utf8, "\xef\xbb\xbf<n/>", 7));
+	CHECK(test_save_narrow(doc, flags, encoding_utf16_be, "\xfe\xff\x00<\x00n\x00/\x00>", 10));
+	CHECK(test_save_narrow(doc, flags, encoding_utf16_le, "\xff\xfe<\x00n\x00/\x00>\x00", 10));
+	CHECK(test_save_narrow(doc, flags, encoding_utf32_be, "\x00\x00\xfe\xff\x00\x00\x00<\x00\x00\x00n\x00\x00\x00/\x00\x00\x00>", 20));
+	CHECK(test_save_narrow(doc, flags, encoding_utf32_le, "\xff\xfe\x00\x00<\x00\x00\x00n\x00\x00\x00/\x00\x00\x00>\x00\x00\x00", 20));
+	CHECK(test_save_narrow(doc, flags, encoding_latin1, "<n/>", 4));
 
 	// encodings synonyms
 	CHECK(save_narrow(doc, flags, encoding_utf16) == save_narrow(doc, flags, (is_little_endian() ? encoding_utf16_le : encoding_utf16_be)));
@@ -436,7 +662,7 @@ TEST_XML(document_save_declaration, "<node/>")
 {
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, get_native_encoding());
+	doc.save(writer, STR(""), format_default, get_native_encoding());
 
 	CHECK(writer.as_string() == STR("<?xml version=\"1.0\"?>\n<node />\n"));
 }
@@ -447,7 +673,7 @@ TEST(document_save_declaration_empty)
 
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, get_native_encoding());
+	doc.save(writer, STR(""), format_default, get_native_encoding());
 
 	CHECK(writer.as_string() == STR("<?xml version=\"1.0\"?>\n"));
 }
@@ -458,7 +684,7 @@ TEST_XML(document_save_declaration_present_first, "<node/>")
 
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, get_native_encoding());
+	doc.save(writer, STR(""), format_default, get_native_encoding());
 
 	CHECK(writer.as_string() == STR("<?xml encoding=\"utf8\"?>\n<node />\n"));
 }
@@ -470,7 +696,7 @@ TEST_XML(document_save_declaration_present_second, "<node/>")
 
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, get_native_encoding());
+	doc.save(writer, STR(""), format_default, get_native_encoding());
 
 	CHECK(writer.as_string() == STR("<!--text-->\n<?xml encoding=\"utf8\"?>\n<node />\n"));
 }
@@ -481,7 +707,7 @@ TEST_XML(document_save_declaration_present_last, "<node/>")
 
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, get_native_encoding());
+	doc.save(writer, STR(""), format_default, get_native_encoding());
 
 	// node writer only looks for declaration before the first element child
 	CHECK(writer.as_string() == STR("<?xml version=\"1.0\"?>\n<node />\n<?xml encoding=\"utf8\"?>\n"));
@@ -491,15 +717,24 @@ TEST_XML(document_save_declaration_latin1, "<node/>")
 {
 	xml_writer_string writer;
 
-	doc.save(writer, STR(""), pugi::format_default, encoding_latin1);
+	doc.save(writer, STR(""), format_default, encoding_latin1);
 
 	CHECK(writer.as_narrow() == "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<node />\n");
 }
 
+TEST_XML(document_save_declaration_raw, "<node/>")
+{
+	xml_writer_string writer;
+
+	doc.save(writer, STR(""), format_raw, get_native_encoding());
+
+	CHECK(writer.as_string() == STR("<?xml version=\"1.0\"?><node/>"));
+}
+
 struct temp_file
 {
 	char path[512];
-	
+
 	temp_file()
 	{
 		static int index = 0;
@@ -520,8 +755,8 @@ TEST_XML(document_save_file, "<node/>")
 
 	CHECK(doc.save_file(f.path));
 
-	CHECK(doc.load_file(f.path, pugi::parse_default | pugi::parse_declaration));
-	CHECK_NODE(doc, STR("<?xml version=\"1.0\"?><node />"));
+	CHECK(doc.load_file(f.path, parse_default | parse_declaration));
+	CHECK_NODE(doc, STR("<?xml version=\"1.0\"?><node/>"));
 }
 
 TEST_XML(document_save_file_wide, "<node/>")
@@ -534,8 +769,8 @@ TEST_XML(document_save_file_wide, "<node/>")
 
 	CHECK(doc.save_file(wpath));
 
-	CHECK(doc.load_file(f.path, pugi::parse_default | pugi::parse_declaration));
-	CHECK_NODE(doc, STR("<?xml version=\"1.0\"?><node />"));
+	CHECK(doc.load_file(f.path, parse_default | parse_declaration));
+	CHECK_NODE(doc, STR("<?xml version=\"1.0\"?><node/>"));
 }
 
 TEST_XML(document_save_file_error, "<node/>")
@@ -547,10 +782,10 @@ TEST_XML(document_save_file_text, "<node/>")
 {
 	temp_file f;
 
-	CHECK(doc.save_file(f.path, STR(""), pugi::format_no_declaration | pugi::format_save_file_text));
+	CHECK(doc.save_file(f.path, STR(""), format_no_declaration | format_save_file_text));
     CHECK(test_file_contents(f.path, "<node />\n", 9) || test_file_contents(f.path, "<node />\r\n", 10));
 
-	CHECK(doc.save_file(f.path, STR(""), pugi::format_no_declaration));
+	CHECK(doc.save_file(f.path, STR(""), format_no_declaration));
     CHECK(test_file_contents(f.path, "<node />\n", 9));
 }
 
@@ -562,10 +797,10 @@ TEST_XML(document_save_file_wide_text, "<node/>")
 	wchar_t wpath[sizeof(f.path)];
 	std::copy(f.path, f.path + strlen(f.path) + 1, wpath + 0);
 
-	CHECK(doc.save_file(wpath, STR(""), pugi::format_no_declaration | pugi::format_save_file_text));
+	CHECK(doc.save_file(wpath, STR(""), format_no_declaration | format_save_file_text));
     CHECK(test_file_contents(f.path, "<node />\n", 9) || test_file_contents(f.path, "<node />\r\n", 10));
 
-	CHECK(doc.save_file(wpath, STR(""), pugi::format_no_declaration));
+	CHECK(doc.save_file(wpath, STR(""), format_no_declaration));
     CHECK(test_file_contents(f.path, "<node />\n", 9));
 }
 
@@ -591,39 +826,39 @@ TEST_XML(document_save_file_wide_leak, "<node/>")
 
 TEST(document_load_buffer)
 {
-	const pugi::char_t text[] = STR("<?xml?><node/>");
+	const char_t text[] = STR("<?xml?><node/>");
 
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_buffer(text, sizeof(text)));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_buffer_inplace)
 {
-	pugi::char_t text[] = STR("<?xml?><node/>");
+	char_t text[] = STR("<?xml?><node/>");
 
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_buffer_inplace(text, sizeof(text)));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_load_buffer_inplace_own)
 {
 	allocation_function alloc = get_memory_allocation_function();
 
-	size_t size = strlen("<?xml?><node/>") * sizeof(pugi::char_t);
+	size_t size = strlen("<?xml?><node/>") * sizeof(char_t);
 
-	pugi::char_t* text = static_cast<pugi::char_t*>(alloc(size));
+	char_t* text = static_cast<char_t*>(alloc(size));
 	CHECK(text);
 
 	memcpy(text, STR("<?xml?><node/>"), size);
 
-	pugi::xml_document doc;
+	xml_document doc;
 
 	CHECK(doc.load_buffer_inplace_own(text, size));
-	CHECK_NODE(doc, STR("<node />"));
+	CHECK_NODE(doc, STR("<node/>"));
 }
 
 TEST(document_parse_result_bool)
@@ -672,7 +907,7 @@ inline void check_utftest_document(const xml_document& doc)
 	CHECK(static_cast<unsigned int>(doc.last_child().last_child().name()[0]) >= 0x80);
 
 	// check magic string
-	const pugi::char_t* v = doc.last_child().child(STR("Heavy")).previous_sibling().child_value();
+	const char_t* v = doc.last_child().child(STR("Heavy")).previous_sibling().child_value();
 
 #ifdef PUGIXML_WCHAR_MODE
 	CHECK(v[0] == 0x4e16 && v[1] == 0x754c && v[2] == 0x6709 && v[3] == 0x5f88 && v[4] == 0x591a && v[5] == wchar_cast(0x8bed) && v[6] == wchar_cast(0x8a00));
@@ -915,7 +1150,7 @@ TEST(document_contents_preserve_latin1)
 		{
 			// parse into document (preserve comments, declaration and whitespace pcdata)
 			xml_document doc;
-			CHECK(doc.load_buffer(files[src].data, files[src].size, parse_default | parse_ws_pcdata | parse_declaration | parse_comments, files[src].encoding));
+			CHECK(doc.load_buffer(files[src].data, files[src].size, parse_default | parse_ws_pcdata | parse_declaration | parse_comments));
 
 			// compare saved document with the original (raw formatting, without extra declaration, write bom if it was in original file)
 			CHECK(test_save_narrow(doc, format_raw | format_no_declaration | format_write_bom, files[dst].encoding, files[dst].data, files[dst].size));
@@ -1006,7 +1241,7 @@ TEST(document_load_buffer_empty)
 		CHECK(doc.load_buffer_inplace(buffer, 0, parse_default, encoding).status == status_no_document_element && !doc.first_child());
 		CHECK(doc.load_buffer_inplace(0, 0, parse_default, encoding).status == status_no_document_element && !doc.first_child());
 
-		void* own_buffer = pugi::get_memory_allocation_function()(1);
+		void* own_buffer = get_memory_allocation_function()(1);
 
 		CHECK(doc.load_buffer_inplace_own(own_buffer, 0, parse_default, encoding).status == status_no_document_element && !doc.first_child());
 		CHECK(doc.load_buffer_inplace_own(0, 0, parse_default, encoding).status == status_no_document_element && !doc.first_child());
@@ -1042,7 +1277,7 @@ TEST(document_load_buffer_empty_fragment)
 		CHECK(doc.load_buffer_inplace(buffer, 0, parse_fragment, encoding) && !doc.first_child());
 		CHECK(doc.load_buffer_inplace(0, 0, parse_fragment, encoding) && !doc.first_child());
 
-		void* own_buffer = pugi::get_memory_allocation_function()(1);
+		void* own_buffer = get_memory_allocation_function()(1);
 
 		CHECK(doc.load_buffer_inplace_own(own_buffer, 0, parse_fragment, encoding) && !doc.first_child());
 		CHECK(doc.load_buffer_inplace_own(0, 0, parse_fragment, encoding) && !doc.first_child());
@@ -1092,7 +1327,7 @@ TEST(document_progressive_truncation)
 			bool result = doc.load_buffer_inplace(truncated_data, i, parse_default | parse_fragment);
 
 			// some truncate locations are parseable - those that come after declaration, declaration + doctype, declaration + doctype + comment and eof
-			CHECK(((i - 21) < 3 || (i - 66) < 3 || (i - 95) < 3 || i == original_size) ? result : !result);
+			CHECK(((i >= 21 && i <= 23) || (i >= 66 && i <= 68) || (i >= 95 && i <= 97) || i == original_size) ? result : !result);
 		}
 	}
 
@@ -1158,7 +1393,7 @@ TEST(document_load_exceptions)
 
     try
     {
-        pugi::xml_document doc;
+        xml_document doc;
         if (!doc.load_string(STR("<node attribute='value"))) throw std::bad_alloc();
 
         CHECK_FORCE_FAIL("Expected parsing failure");
@@ -1196,7 +1431,7 @@ TEST_XML(document_reset, "<node><child/></node>")
 
     CHECK(doc.load_string(STR("<node/>")));
     CHECK(doc.first_child());
-    CHECK_NODE(doc, STR("<node />"));
+    CHECK_NODE(doc, STR("<node/>"));
 
     doc.reset();
     CHECK(!doc.first_child());
@@ -1220,12 +1455,12 @@ TEST_XML(document_reset_copy, "<node><child/></node>")
 
     doc2.reset(doc);
 
-    CHECK_NODE(doc2, STR("<node><child /></node>"));
+    CHECK_NODE(doc2, STR("<node><child/></node>"));
     CHECK(doc.first_child() != doc2.first_child());
 
     doc.reset(doc2);
 
-    CHECK_NODE(doc, STR("<node><child /></node>"));
+    CHECK_NODE(doc, STR("<node><child/></node>"));
     CHECK(doc.first_child() != doc2.first_child());
 
     CHECK(doc.first_child().offset_debug() == -1);
@@ -1233,7 +1468,7 @@ TEST_XML(document_reset_copy, "<node><child/></node>")
 
 TEST_XML(document_reset_copy_self, "<node><child/></node>")
 {
-    CHECK_NODE(doc, STR("<node><child /></node>"));
+    CHECK_NODE(doc, STR("<node><child/></node>"));
 
     doc.reset(doc);
 
@@ -1349,7 +1584,7 @@ TEST(document_alignment)
 		xml_document* doc = new (buf + offset) xml_document;
 
 		CHECK(doc->load_string(STR("<node />")));
-		CHECK_NODE(*doc, STR("<node />"));
+		CHECK_NODE(*doc, STR("<node/>"));
 
 		doc->~xml_document();
 	}
@@ -1389,9 +1624,187 @@ TEST(document_convert_out_of_memory)
 	}
 }
 
-TEST(document_deprecated_load)
+#ifdef PUGIXML_HAS_MOVE
+TEST_XML(document_move_ctor, "<node1/><node2/>")
 {
-	xml_document doc;
-	CHECK(doc.load(STR("<node/>")));
-	CHECK_NODE(doc, STR("<node />"));
+	xml_document other = std::move(doc);
+
+	CHECK(doc.first_child().empty());
+
+	CHECK_STRING(other.first_child().name(), STR("node1"));
+	CHECK(other.first_child().parent() == other);
+
+	CHECK_STRING(other.last_child().name(), STR("node2"));
+	CHECK(other.last_child().parent() == other);
 }
+
+TEST_XML(document_move_assign, "<node1/><node2/>")
+{
+	xml_document other;
+	CHECK(other.load_string(STR("<node3/>")));
+
+	other = std::move(doc);
+
+	CHECK(doc.first_child().empty());
+
+	CHECK_STRING(other.first_child().name(), STR("node1"));
+	CHECK(other.first_child().parent() == other);
+
+	CHECK_STRING(other.last_child().name(), STR("node2"));
+	CHECK(other.last_child().parent() == other);
+}
+
+TEST_XML(document_move_zero_alloc, "<node1/><node2/>")
+{
+	test_runner::_memory_fail_threshold = 1;
+
+	xml_document other = std::move(doc);
+
+	CHECK(doc.first_child().empty());
+
+	CHECK_STRING(other.first_child().name(), STR("node1"));
+	CHECK(other.first_child().parent() == other);
+
+	CHECK_STRING(other.last_child().name(), STR("node2"));
+	CHECK(other.last_child().parent() == other);
+}
+
+TEST(document_move_append_buffer)
+{
+	xml_document* doc = new xml_document();
+	CHECK(doc->load_string(STR("<node1 attr1='value1'><node2/></node1>")));
+	CHECK(doc->child(STR("node1")).append_buffer("<node3/>", 8));
+	CHECK(doc->child(STR("node1")).append_buffer("<node4/>", 8));
+
+	xml_document other = std::move(*doc);
+	delete doc;
+
+	CHECK(other.child(STR("node1")).append_buffer("<node5/>", 8));
+	CHECK(other.child(STR("node1")).append_buffer("<node6/>", 8));
+
+	CHECK_NODE(other, STR("<node1 attr1=\"value1\"><node2/><node3/><node4/><node5/><node6/></node1>"));
+}
+
+TEST(document_move_append_child)
+{
+	xml_document* doc = new xml_document();
+	CHECK(doc->load_string(STR("<node1 attr1='value1'><node2/></node1>")));
+
+	xml_document other = std::move(*doc);
+	delete doc;
+
+	for (int i = 0; i < 3000; ++i)
+		other.child(STR("node1")).append_child(STR("node"));
+
+	for (int i = 0; i < 3000; ++i)
+		other.child(STR("node1")).remove_child(other.child(STR("node1")).last_child());
+
+	CHECK_NODE(other, STR("<node1 attr1=\"value1\"><node2/></node1>"));
+
+	other.remove_child(other.first_child());
+
+	CHECK(!other.first_child());
+}
+
+TEST(document_move_empty)
+{
+	xml_document* doc = new xml_document();
+	xml_document other = std::move(*doc);
+	delete doc;
+}
+
+TEST(document_move_large)
+{
+	xml_document* doc = new xml_document();
+
+	xml_node dn = doc->append_child(STR("node"));
+
+	for (int i = 0; i < 3000; ++i)
+		dn.append_child(STR("child"));
+
+	xml_document other = std::move(*doc);
+	delete doc;
+
+	xml_node on = other.child(STR("node"));
+
+	for (int i = 0; i < 3000; ++i)
+		CHECK(on.remove_child(on.first_child()));
+
+	CHECK(!on.first_child());
+}
+
+TEST_XML(document_move_buffer, "<node1/><node2/>")
+{
+	CHECK(doc.child(STR("node2")).offset_debug() == 9);
+
+	xml_document other = std::move(doc);
+
+	CHECK(other.child(STR("node2")).offset_debug() == 9);
+}
+
+TEST_XML(document_move_append_child_zero_alloc, "<node1/><node2/>")
+{
+	test_runner::_memory_fail_threshold = 1;
+
+	xml_document other = std::move(doc);
+
+	CHECK(other.append_child(STR("node3")));
+
+	CHECK_NODE(other, STR("<node1/><node2/><node3/>"));
+}
+
+TEST(document_move_empty_zero_alloc)
+{
+	xml_document* docs = new xml_document[32];
+
+	test_runner::_memory_fail_threshold = 1;
+
+	for (int i = 1; i < 32; ++i)
+		docs[i] = std::move(docs[i-1]);
+
+	delete[] docs;
+}
+
+#ifndef PUGIXML_COMPACT
+TEST(document_move_repeated_zero_alloc)
+{
+	xml_document docs[32];
+
+	CHECK(docs[0].load_string(STR("<node><child/></node>")));
+
+	test_runner::_memory_fail_threshold = 1;
+
+	for (int i = 1; i < 32; ++i)
+		docs[i] = std::move(docs[i-1]);
+
+	for (int i = 0; i < 31; ++i)
+		CHECK(!docs[i].first_child());
+
+	CHECK_NODE(docs[31], STR("<node><child/></node>"));
+}
+#endif
+
+#ifdef PUGIXML_COMPACT
+TEST(document_move_compact_fail)
+{
+	xml_document docs[32];
+
+	CHECK(docs[0].load_string(STR("<node><child/></node>")));
+
+	test_runner::_memory_fail_threshold = 1;
+
+	int safe_count = 21;
+
+	for (int i = 1; i <= safe_count; ++i)
+		docs[i] = std::move(docs[i-1]);
+
+	CHECK_ALLOC_FAIL(docs[safe_count+1] = std::move(docs[safe_count]));
+
+	for (int i = 0; i < safe_count; ++i)
+		CHECK(!docs[i].first_child());
+
+	CHECK_NODE(docs[safe_count], STR("<node><child/></node>"));
+	CHECK(!docs[safe_count+1].first_child());
+}
+#endif
+#endif
