@@ -1,12 +1,14 @@
 #define PUGIXML_HEADER_ONLY
 #define pugi pugih
 
-#include "common.hpp"
+#include "test.hpp"
 
 // Check header guards
 #include "../src/pugixml.hpp"
 #include "../src/pugixml.hpp"
 
+using namespace pugi;
+
 TEST(header_only_2)
 {
 	xml_document doc;
